@@ -1,23 +1,24 @@
 from __future__ import annotations
 
+import contextlib
 import logging
 import os
 import socket
 import sqlite3
+import tempfile
+from collections.abc import Iterable
+from collections.abc import Iterator
 from datetime import date
 from datetime import datetime
 from datetime import time
 from datetime import timedelta
+from datetime import timezone
 from functools import cached_property
 from os.path import normpath
 from os.path import split
-from typing import Any
-from typing import Iterable
 from uuid import uuid4
 
 import icalendar
-import pytz
-from atomicwrites import AtomicWriter
 from dateutil.rrule import rrulestr
 from dateutil.tz import tzlocal
 
@@ -61,16 +62,13 @@ class Todo:
         new: bool = False,
         list: TodoList | None = None,
     ) -> None:
-        """
-        Creates a new todo using `todo` as a source.
+        """Creates a new todo using `todo` as a source.
 
-        :param str filename: The name of the file for this todo. Defaults to
-            the <uid>.ics
-        :param mtime int: The last modified time for the file backing this
-            Todo.
-        :param bool new: Indicate that a new Todo is being created and should
-            be populated with default values.
-        :param TodoList list: The list to which this Todo belongs.
+        :param filename: The name of the file for this todo. Defaults to the <uid>.ics
+        :param int: The last modified time for the file backing this Todo.
+        :param new: Indicate that a new Todo is being created and should be populated
+            with default values.
+        :param list: The list to which this Todo belongs.
         """
         self.list = list
         now = datetime.now(LOCAL_TIMEZONE)
@@ -171,33 +169,33 @@ class Todo:
             if value is None:
                 v = ""
             else:
-                assert isinstance(
-                    value, str
-                ), f"Got {type(value)} for {name} where str was expected"
+                assert isinstance(value, str), (
+                    f"Got {type(value)} for {name} where str was expected"
+                )
 
         if name in Todo.STRING_FIELDS:
             if value is None:
                 v = ""
             else:
-                assert isinstance(
-                    value, str
-                ), f"Got {type(value)} for {name} where str was expected"
+                assert isinstance(value, str), (
+                    f"Got {type(value)} for {name} where str was expected"
+                )
 
         if name in Todo.INT_FIELDS:
             if value is None:
                 v = 0
             else:
-                assert isinstance(
-                    value, int
-                ), f"Got {type(value)} for {name} where int was expected"
+                assert isinstance(value, int), (
+                    f"Got {type(value)} for {name} where int was expected"
+                )
 
         if name in Todo.LIST_FIELDS:
             if value is None:
                 v = []
             else:
-                assert isinstance(
-                    value, list
-                ), f"Got {type(value)} for {name} where list was expected"
+                assert isinstance(value, list), (
+                    f"Got {type(value)} for {name} where list was expected"
+                )
 
         return object.__setattr__(self, name, v)
 
@@ -213,6 +211,7 @@ class Todo:
         if not dt:
             return None
 
+        assert self.rrule is not None, "applying recurrence to todo without rrule"
         recurrence = rrulestr(self.rrule, dtstart=dt)
 
         if isinstance(dt, date) and not isinstance(dt, datetime):
@@ -261,6 +260,27 @@ class Todo:
         self.status = "CANCELLED"
 
 
+@contextlib.contextmanager
+def atomic_write(dest, overwrite=False):
+    fd, src = tempfile.mkstemp(prefix=os.path.basename(dest), dir=os.path.dirname(dest))
+    file = os.fdopen(fd, mode='wb')
+
+    try:
+        yield file
+    except Exception:
+        os.unlink(src)
+        raise
+    else:
+        file.flush()
+        file.close()
+
+        if overwrite:
+            os.rename(src, dest)
+        else:
+            os.link(src, dest)
+            os.unlink(src)
+
+
 class VtodoWriter:
     """Writes a Todo as a VTODO file."""
 
@@ -306,28 +326,7 @@ class VtodoWriter:
         if not dt.tzinfo:
             dt = dt.replace(tzinfo=LOCAL_TIMEZONE)
 
-        return dt.astimezone(pytz.UTC)
-
-    def serialize_field(self, name: str, value: Any) -> int | str | date:
-        if name in Todo.RRULE_FIELDS:
-            return icalendar.vRecur.from_ical(value)
-        if name in Todo.DATETIME_FIELDS:
-            return self.normalize_datetime(value)
-        if name in Todo.LIST_FIELDS:
-            return value
-        if name in Todo.INT_FIELDS:
-            return int(value)
-        if name in Todo.STRING_FIELDS:
-            return value
-
-        raise Exception(f"Unknown field {name} serialized.")
-
-    def set_field(self, name: str, value: Any) -> None:
-        # If serialized value is None:
-        self.vtodo.pop(name)
-        if value:
-            logger.debug("Setting field %s to %s.", name, value)
-            self.vtodo.add(name, value)
+        return dt.astimezone(timezone.utc)
 
     def serialize(self, original: icalendar.Todo = None) -> icalendar.Todo:
         """Serialize a Todo into a VTODO."""
@@ -338,10 +337,24 @@ class VtodoWriter:
         for source, target in self.FIELD_MAP.items():
             self.vtodo.pop(target)
             if getattr(self.todo, source):
-                self.set_field(
-                    target,
-                    self.serialize_field(source, getattr(self.todo, source)),
-                )
+                raw = getattr(self.todo, source)
+                if source in Todo.RRULE_FIELDS:
+                    value = icalendar.vRecur.from_ical(raw)
+                elif source in Todo.DATETIME_FIELDS:
+                    value = self.normalize_datetime(raw)
+                elif source in Todo.LIST_FIELDS:
+                    value = raw
+                elif source in Todo.INT_FIELDS:
+                    value = int(raw)
+                elif source in Todo.STRING_FIELDS:
+                    value = raw
+                else:
+                    raise Exception(f"Unknown field {source} serialized.")
+
+                self.vtodo.pop(target)  # Removes if None, avoids dupes
+                if value:
+                    logger.debug("Setting field %s to %s.", target, value)
+                    self.vtodo.add(target, value)
 
         return self.vtodo
 
@@ -371,7 +384,7 @@ class VtodoWriter:
                 if component.get("uid", None) == self.todo.uid:
                     cal.subcomponents[index] = vtodo
 
-        with AtomicWriter(path, "wb", overwrite=True).open() as f:
+        with atomic_write(path, overwrite=True) as f:
             f.write(cal.to_ical())
 
     def _write_new(self, path: str) -> icalendar.Todo:
@@ -380,8 +393,8 @@ class VtodoWriter:
         c = icalendar.Calendar()
         c.add_component(vtodo)
 
-        with AtomicWriter(path, "wb").open() as f:
-            c.add("prodid", "io.barrera.todoman")
+        with atomic_write(path) as f:
+            c.add("prodid", "nl.whynothugo.todoman")
             c.add("version", "2.0")
             f.write(c.to_ical())
 
@@ -722,7 +735,7 @@ class Cache:
         start: tuple[bool, datetime] | None = None,
         startable: bool = False,
         status: str = "NEEDS-ACTION,IN-PROCESS",
-    ) -> Iterable[Todo]:
+    ) -> Iterator[Todo]:
         """
         Returns filtered cached todos, in a specified order.
 
@@ -742,8 +755,8 @@ class Cache:
         :param reverse: Reverse the order of the todos after sorting.
         :param due: Return only todos due within ``due`` hours.
         :param priority: Only return todos with priority at least as high as specified.
-        :param  start: Return only todos before/after ``start`` date
-        :param  status: Return only todos with any of the given statuses.
+        :param start: Return only todos before/after ``start`` date
+        :param status: Return only todos with any of the given statuses.
         :return: A sorted, filtered list of todos.
         """
         extra_where = []
@@ -898,7 +911,7 @@ class Cache:
         todo.rrule = row["rrule"]
         return todo
 
-    def lists(self) -> Iterable[TodoList]:
+    def lists(self) -> Iterator[TodoList]:
         result = self._conn.execute("SELECT * FROM lists")
         for row in result:
             yield TodoList(
@@ -963,7 +976,7 @@ class Cache:
         result = self._conn.execute("SELECT path, mtime FROM files")
         for row in result:
             path, mtime = row["path"], row["mtime"]
-            if paths_to_mtime.get(path, None) != mtime:
+            if paths_to_mtime.get(path) != mtime:
                 self.expire_file(path)
 
     def expire_file(self, path: str) -> None:
@@ -1075,13 +1088,13 @@ class Database:
 
         self.cache.save_to_disk()
 
-    def todos(self, **kwargs: Any) -> Iterable[Todo]:
+    def todos(self, **kwargs) -> Iterator[Todo]:
         return self.cache.todos(**kwargs)
 
-    def todo(self, id: int, **kwargs: Any) -> Todo:
+    def todo(self, id: int, **kwargs) -> Todo:
         return self.cache.todo(id, **kwargs)
 
-    def lists(self) -> Iterable[TodoList]:
+    def lists(self) -> Iterator[TodoList]:
         return self.cache.lists()
 
     def move(self, todo: Todo, new_list: TodoList, from_list: TodoList) -> None:
